import numpy as np
class MaxAbsScaler:
    def __init__(self):
        self.Absmax=None
    
    def fit(self,x):
        self.Absmax=np.max(abs(x),axis=0)
        self.scale_=np.where(self.Absmax==0,1,self.Absmax)
        return self
    
    def fit_transform(self,x):
        self.fit(x)
        return x/self.scale_
    
    def transform(self,x):
        return x/self.scale_
    



import numpy as np
from collections import Counter
class SimpleImputer:
    def __init__(self,fill_value='mean'):
        self.fill_value=fill_value

    def fill(self,X):
        X=np.array(X,dtype=float)
        if self.fill_value=='mean':
            mean=np.nanmean(X)
            X[np.isnan(X)]=mean

        elif self.fill_value=='mode':
            non_nan=X[~np.isnan(X)]
            count=Counter(non_nan)
            mode=count.most_common(1)[0][0]
            X[np.isnan(X)]=mode
            

        elif self.fill_value=='median':
            median=np.nanmedian(X)
            X[np.isnan(X)]=median     

        elif isinstance(self.fill_value,(float,int)):
            X[np.isnan(X)]=self.fill_value

        else:
            raise ValueError(f"Invalid Fill value. Choose from 'mean', 'median', 'mode', or a number.")
        
        return X
        




        


import numpy as np
class LinearRegression:
    def __init__(self):
        self.coef=None
        self.intercept=None

    def fit(self,X_train,y_train):
        X_train=np.insert(X_train,0,1,axis=1)
        beta=np.linalg.inv(np.dot(X_train.T,X_train)).dot(X_train.T).dot(y_train)
        self.intercept=beta[0]
        self.coef=beta[1:]

    def predict(self,X_test):
        return np.dot(X_test,self.coef)+self.intercept
        
        


import numpy as np
class GDRidge:
    def __init__(self,epochs,learning_rate,alpha):
        self.learning_rate=learning_rate
        self.epochs=epochs
        self.alpha=alpha
        self.intercept=None
        self.coef=None

    def fit(self,X_train,y_train):
        self.coef=np.ones(X_train.shape[1])
        self.intercept=0
        thetha=np.insert(self.coef,0,self.intercept)
        X_train=np.insert(X_train,0,1,axis=1)

        for i in range(self.epochs):
            thetha_der=np.dot(X_train.T,X_train).dot(thetha)-np.dot(X_train.T,y_train)+self.alpha*thetha
            thetha=thetha-self.learning_rate*thetha_der
        self.coef=thetha[1:]
        self.intercept=thetha[0]
    
    def predict(self,X_test):
        return np.dot(X_test,self.coef)+self.intercept

        


import numpy as np
class LogisticRegression:
    def __init__(self,epochs=1000,learning_rate=0.1,patience=20,min_del=1e-4):
        self.w=None
        self.b=None
        self.no_of_class=None
        self.epochs=epochs
        self.learning_rate=learning_rate
        self.patience=patience
        self.min_del=min_del

    def softmax(self,z):
        num=np.exp(z-np.max(z,axis=1,keepdims=True))
        return num/np.sum(num,axis=1,keepdims=True)
    
    def one_hot(self,y):
        one_hot=np.zeros((y.shape[0],self.no_of_class))
        for i in range(y.shape[0]):
            one_hot[i,y[i]]=1
        return one_hot
    
    def compute_loss(self,y_true,y_pred):
        m=y_true.shape[0]
        return -np.sum(y_true*np.log(y_pred+1e-4))/m
    

    def fit(self,X,y):
        # m-row n-column
        # y is m-row 1-column
        m,n=X.shape
        # number of classes
        self.no_of_class=np.max(y)+1
        # weight n*k(no. of class)
        self.w=np.zeros((n,self.no_of_class))
        # b 1*k
        self.b=np.zeros((1,self.no_of_class))
        epoch_no=0
        best_loss=float('inf')
        for epoch in range(self.epochs):
            # z m*k
            z=np.dot(X,self.w)+self.b
            # y_hat m*k
            y_hat=self.softmax(z)
            # m*c
            y_true=self.one_hot(y)

            loss=self.compute_loss(y_true,y_hat)
            if best_loss-loss>=self.min_del:
                best_loss=loss
                epoch_no=0
                
            else:
                epoch_no+=1
                
            if epoch_no>=self.patience:
                print(f"No improvement since {self.patience} epochs.Performing early stoping at {epoch}...")
                break


            # applying gradient descent
            self.w-=self.learning_rate*np.dot(X.T,(y_hat-y_true))/m
            self.b-=np.sum(self.learning_rate*(y_hat-y_true),keepdims=True)/m
        
    def predict(self,X):
        z=np.dot(X,self.w)+self.b
        prob=self.softmax(z)
        return np.argmax(prob,axis=1)
    
    def predict_prob(self,X):
        z=np.dot(X,self.w)+self.b
        return self.softmax(z)

    
        


import numpy as np
class MeanScaler:
    def __init__(self):
        self.mean=None
    
    def fit(self,X_train):
        self.mean=X_train.mean(axis=0)   
        return self
    
    def fit_transform(self,X_train):
        self.fit(X_train)
        return X_train-self.mean
    
    def transform(self,X_test):
        return X_test-self.mean





import numpy as np
class OLSRidge:
    def __init__(self,alpha=0.1):
        self.coef=None
        self.intercept=None
        self.alpha=alpha

    def fit(self,X_train,y_train):
        X_train=np.insert(X_train,0,1,axis=1)
        I=np.identity(X_train.shape[1])
        I[0][0]=0
        beta=np.linalg.inv(np.dot(X_train.T,X_train)+self.alpha*I).dot(X_train.T).dot(y_train)
        self.intercept=beta[0]
        self.coef=beta[1:]

    def predict(self,X_test):
        return np.dot(X_test,self.coef)+self.intercept
        
        


import numpy as np
class StandardScaler:
    def __init__(self):
        self.mean=None
        self.std=None
    
    def fit(self,X_train):
        self.mean=X_train.mean(axis=0)
        self.std=X_train.std(axis=0)
        self.std[self.std==0]=1
        return self
    
    def fit_transform(self,X_train):
        self.fit(X_train)
        return (X_train-self.mean)/self.std
    
    def transform(self,X_test):
        return (X_test-self.mean)/self.std






import numpy as np
class score:
    def __init__(self,y_test,y_pred):
        
        self.y_test=y_test
        self.y_pred=y_pred
        if self.y_test.shape[0]!=self.y_pred.shape[0]:
            raise ValueError("Given values do not have same dimensions")

    def r2_score(self):
        res=0
        tot=0
        y_mean=self.y_test.mean()

        for i in range(self.y_test.shape[0]):
            res+=(self.y_test[i]-self.y_pred[i])**2
            tot+=(self.y_test[i]-y_mean)**2
        if tot==0:
            raise ValueError("r2 not defined")
        return 1-(res/tot)
    
    def adjusted_r2_score(self,n_features_in):
   
        r=self.r2_score()
        p=n_features_in
        n=self.y_test.shape[0]
        return 1-((1-r)*(n-1)/(n-p-1))
    
    def mse(self):
        n=self.y_test.shape[0]
        mse=0
       
        for i in range(n):
            mse+=(self.y_test[i]-self.y_pred[i])**2
        return mse/n    

    def rmse(self):
        return np.sqrt(self.mse())

    def mae(self):
        n=self.y_test.shape[0]
        mae=0
       
        for i in range(n):
            mae+=abs(self.y_test[i]-self.y_pred[i])
        return mae/n  


from setuptools import setup, find_packages

with open('README.md', 'r', encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='Neurix',                 
    version='0.1.0',                
    author='Avatanshu Gupta',        
    author_email='avatanshugupta@gmail.com',  
    description='EasyMl is a simple ML utility library with preprocessing, regression, classification, and evaluation tools',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/AvatanshuGupta/EasyML.git', 
    packages=find_packages(),       
    python_requires='>=3.8',      
    install_requires=[            
        'numpy'
        
    ],
    classifiers=[                   
        'Programming Language :: Python :: 3',
        'Operating System :: OS Independent',
    ],
)



import numpy as np
class Pearson_corr:
    def __init__(self):
        self.x_mean=None
        self.y_mean=None
        self.x_std=None
        self.y_std=None
        self.feat_len=None
    
    def corr(self,x,y):
        if x.shape[0]!=y.shape[0]:
            raise Exception("Data dimensions not matching")
        self.x_mean=np.mean(x)
        self.y_mean=np.mean(y)
        self.x_std=np.std(x,ddof=1)
        self.y_std=np.std(y,ddof=1)
        self.feat_len=x.shape[0]
        return np.sum((x-self.x_mean)*(y-self.y_mean))/((self.x_std)*(self.y_std)*(self.feat_len-1))
    


import numpy as np
class MinMaxScaler:
    def __init__(self,min=0,max=1):
        self.min=min
        self.max=max
        self.range=max-min
        self.min_val=None
        self.max_val=None
        if self.range<0:
            raise Exception("Min value is greater than max value")
        
    def fit(self,x):
        self.max_val=np.max(x,axis=0)
        self.min_val=np.min(x,axis=0)
        self.scale_=np.where(self.max_val-self.min_val==0,1,self.max_val-self.min_val)
        return self
    
    def fit_transform(self,x):
        self.fit(x)
        return self.min+(x-self.min_val)*(self.range)/self.scale_
    
    def transform(self,x):
        return self.min+(x-self.min_val)*(self.range)/self.scale_


